/*
 * Crypto.java
 * File revision 0
 * Implement abstract crypto system to plug into web applet.
 * (c) 2002 Jacob Lundberg, jacob@chaos2.org
 */

import java.awt.*;

class Crypto {

	/* Globals within Crypto. */
	short keyBuffer[];
	short dataBuffer[];
	short workBuffer[];
	public GUI gui = null;


	/* Reset everything. */
	public void reset() {
		keyBuffer = null;
		dataBuffer = null;
		workBuffer = null;
	}


	/* Print to the GUI debugging box. */
	void WriteGUI(String message) {
		if(gui != null)
			gui.WriteLog(message);
	}


	/* Java needs printf like politics needs morality...
	   (i.e. very very badly but it'll never happen) */
	private String byteHex(short val) {
		char a[] = new char[2];
		a[0] = (char)(val / 16);
		a[1] = (char)(val % 16);
		if(a[0] <= 9)	a[0] += '0';
		else		a[0] += 'a' - 10;
		if(a[1] <= 9)	a[1] += '0';
		else		a[1] += 'a' - 10;
		return(new String(a));
	}
	private byte hexByte(byte hex[]) {
		if(hex.length != 2)
			return(0);
		if(hex[0] >= '0' && hex[0] <= '9')	hex[0] -= '0';
		else					hex[0] -= 'a' - 10;
		if(hex[1] >= '0' && hex[1] <= '9')	hex[1] -= '0';
		else					hex[1] -= 'a' - 10;
		return((byte)((hex[0] * 16 + hex[1]) & 0xff));
	}


	/* Dump a working buffer's contents. */
	public void dumpBuffer(short buffer[]) {
		if(buffer.length > 0) {
			for(int i = 0; i < buffer.length; ++i)
				Write(" " + byteHex(buffer[i]));
		} else {
			Write(" (empty)");
		}
	}


	/* Print the current working result. */
	public void print() {
		Write("Working buffer:");
		dumpBuffer(workBuffer);
		Write("\n");
	}

	/* (Re-)Paint into the display area. */
	public void paint(Graphics gfix) {
	}

	/* Propagate a request to output to console. (This exists for historical reasons.) */
	private void Write(String display) {
		System.out.print(display);
	}


	/* Array redimming functions.  (Very specific to AES.) */
	private void astod(short s[], short d[][], int i, int o) {
		for(int a = 0; a < i * o; ++a)
			d[a % i][a / i] = s[a];
	}
	private void adtos(short s[], short d[][], int i, int o) {
		for(int a = 0; a < i * o; ++a)
			s[a] = d[a % i][a / i];
	}


	/* Conversion of ASCII strings to hexadecimal strings and vise versa. */
	public String ASCIItoHex(String ascii) {
		/*
		 * Convert ASCII text to a 16, 24, or 32 byte hexadecimal string.
		 */
		String hex = new String();
		int i = 0;

		for(; (i < ascii.length()) && (i < 32); ++i)
			hex = new String(hex + byteHex((short)ascii.charAt(i)) + " ");

		for(; (i != 16) && (i != 24) && (i != 32); ++i)
			hex = new String(hex + "00 ");

		hex = hex.trim();
		return(hex);

	}
	public String HextoASCII(String hex) {
		/*
		 * Convert a hexadecimal string to ASCII text.
		 */
		String ascii = new String();
		byte work[] = new byte[hex.length() / 2 + 1];
		byte bite[] = new byte[1];
		int i;

		hex = hex.trim();
		hex = hex.toLowerCase();

		i = 0;
		while(i < hex.length()) {
			bite[0] = (byte)hex.charAt(i);
			if((bite[0] < 'a' || bite[0] > 'f') && (bite[0] < '0' || bite[0] > '9'))
				hex = new String(hex.substring(0, i) + hex.substring(i + 1, hex.length()));
			else
				++i;
		}
		if(i % 2 == 1)
			hex = hex.substring(0, i - 1);

		i = 0;
		while(i < hex.length()) {
			bite = hex.substring(i, i + 2).getBytes();
			work[i / 2] = hexByte(bite);
			i += 2;
		}

		ascii = new String(work);
		return(ascii);

	}


	/* Conversion of Arrays to hexadecimal strings and vise versa. */
	private short[] HextoArray(String hex) {
		byte bite[] = new byte[1];
		short array[];
		int i;

		hex = hex.trim();
		hex = hex.toLowerCase();

		i = 0;
		while(i < hex.length()) {
			bite[0] = (byte)hex.charAt(i);
			if((bite[0] < 'a' || bite[0] > 'f') && (bite[0] < '0' || bite[0] > '9'))
				hex = new String(hex.substring(0, i) + hex.substring(i + 1, hex.length()));
			else
				++i;
		}
		if(i % 2 == 1)
			hex = hex.substring(0, i - 1);

		array = new short[i / 2];
		i = 0;
		while(i < hex.length()) {
			bite = hex.substring(i, i + 2).getBytes();
			array[i / 2] = (short)(hexByte(bite) & 0xff);
			i += 2;
		}

		return(array);

	}
	private String ArraytoHex(short array[]) {
		String hex = new String();
		int i = 0;

		for(; i < array.length; ++i)
			hex = new String(hex + byteHex(array[i]) + " ");

		hex = hex.trim();
		return(hex);

	}


	public String encrypt(String plaintext, String keyvalue, int iter) {
		/*
		 * Perform AES encryption on some data using a key.
		 * Both the data and the key may be 128, 192, or 256 bits.
		 */
		AES Rijndael = new AES();
		int KC, BC, rounds;
		short rk[][][];
		short data[][];
		short key[][];
		short tmp[];

		WriteGUI("-------------------------\nEntering Crypto.encrypt()\n\n");

		tmp = HextoArray(keyvalue);
		KC = tmp.length / 4;
		if(!(KC == 4 || KC == 6 || KC == 8) || tmp.length % 4 != 0) {
			WriteGUI("Invalid key length!\n\n");
			return("Invalid key length!");
		}
		WriteGUI("Input key read as: " + ArraytoHex(tmp) + ".\n");
		WriteGUI("Selected 4x" + String.valueOf(KC) + " key array.\n");
		key = new short[4][KC];
		astod(tmp, key, 4, KC);

		tmp = HextoArray(plaintext);
		BC = tmp.length / 4;
		if(!(BC == 4 || BC == 6 || BC == 8) || tmp.length % 4 != 0) {
			WriteGUI("Invalid data block length!\n\n");
			return("Invalid data block length!");
		}
		WriteGUI("Input data read as: " + ArraytoHex(tmp) + ".\n");
		WriteGUI("Selected 4x" + String.valueOf(BC) + " data array.\n");
		data = new short[4][BC];
		astod(tmp, data, 4, BC);

		rounds = (KC >= BC ? KC : BC) + 6;
		WriteGUI("Selected " + String.valueOf(rounds) + " rounds per encryption.\n\n");
		rk = new short[rounds + 1][4][BC];

		Rijndael.KeySchedule(key, rk, (short)KC, (short)BC, (short)rounds);
		workBuffer = new short[4 * BC];
		for(int i = 0; i <= rounds; ++i) {
			adtos(workBuffer, rk[i], 4, BC);
			WriteGUI("key for round " + String.valueOf(i) + " is: " + ArraytoHex(workBuffer) + ".\n");
		}

		if(iter <= 1) {
			WriteGUI("\nPerforming encryption...\n");
			iter = 1;
		} else {
			WriteGUI("\nPerforming " + String.valueOf(iter) + " iterations of encryption...\n");
		}
		for(; iter > 0; --iter)
			stepEncrypt(data, rk, (short)BC, (short)rounds);
		WriteGUI("Encryption completed.\n\n");

		tmp = new short[4 * BC];
		adtos(tmp, data, 4, BC);
		WriteGUI("Resulting ciphertext is: " + ArraytoHex(tmp) + ".\n\n");

		return(ArraytoHex(tmp));

	}


	public String decrypt(String ciphertext, String keyvalue, int iter) {
		/*
		 * Perform AES decryption on some data using a key.
		 * Both the data and the key may be 128, 192, or 256 bits.
		 */
		AES Rijndael = new AES();
		int KC, BC, rounds;
		short rk[][][];
		short data[][];
		short key[][];
		short tmp[];

		WriteGUI("-------------------------\nEntering Crypto.decrypt()\n\n");

		tmp = HextoArray(keyvalue);
		KC = tmp.length / 4;
		if(!(KC == 4 || KC == 6 || KC == 8) || tmp.length % 4 != 0) {
			WriteGUI("Invalid key length!\n\n");
			return("Invalid key length!");
		}
		WriteGUI("Input key read as: " + ArraytoHex(tmp) + ".\n");
		WriteGUI("Selected 4x" + String.valueOf(KC) + " key array.\n");
		key = new short[4][KC];
		astod(tmp, key, 4, KC);

		tmp = HextoArray(ciphertext);
		BC = tmp.length / 4;
		if(!(BC == 4 || BC == 6 || BC == 8) || tmp.length % 4 != 0) {
			WriteGUI("Invalid data block length!\n\n");
			return("Invalid data block length!");
		}
		WriteGUI("Input data read as: " + ArraytoHex(tmp) + ".\n");
		WriteGUI("Selected 4x" + String.valueOf(BC) + " data array.\n");
		data = new short[4][BC];
		astod(tmp, data, 4, BC);

		rounds = (KC >= BC ? KC : BC) + 6;
		WriteGUI("Selected " + String.valueOf(rounds) + " rounds per decryption.\n\n");
		rk = new short[rounds + 1][4][BC];

		Rijndael.KeySchedule(key, rk, (short)KC, (short)BC, (short)rounds);
		workBuffer = new short[4 * BC];
		for(int i = 0; i <= rounds; ++i) {
			adtos(workBuffer, rk[i], 4, BC);
			WriteGUI("key for round " + String.valueOf(i) + " is: " + ArraytoHex(workBuffer) + ".\n");
		}

		if(iter <= 1) {
			WriteGUI("\nPerforming decryption...\n");
			iter = 1;
		} else {
			WriteGUI("\nPerforming " + String.valueOf(iter) + " iterations of decryption...\n");
		}
		for(; iter > 0; --iter)
			stepDecrypt(data, rk, (short)BC, (short)rounds);
		WriteGUI("Decryption completed.\n\n");

		tmp = new short[4 * BC];
		adtos(tmp, data, 4, BC);
		WriteGUI("Resulting plaintext is: " + ArraytoHex(tmp) + ".\n\n");

		return(ArraytoHex(tmp));

	}


	public static void main(String argv[]) {
		new Crypto().runMain();
	}


	public void runMain() {
		/*
		 * Primary list of operations to execute.
		 */
		AES Rijndael = new AES();
		int KC, BC, rounds;

		reset();

		// For now, just hard code application values for testing purposes.
		short tkb[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
		short tdb[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
		dataBuffer = tdb;
		keyBuffer = tkb;

		/*
		 * Compute the data and key sizes and the number of rounds.
		 */
		KC = keyBuffer.length / 4;
		if(keyBuffer.length % 4 != 0) {
			Write("Invalid key length!\n");
			return;
		}
		Write("Selected KC = " + new Integer(KC) + ".\n");

		BC = dataBuffer.length / 4;
		if(dataBuffer.length % 4 != 0) {
			Write("Invalid data block length!\n");
			return;
		}
		Write("Selected BC = " + new Integer(BC) + ".\n");

		rounds = (KC >= BC ? KC : BC) + 6;
		Write("Selected " + new Integer(rounds) + " rounds.\n");

		short rk[][][] = new short[rounds + 1][4][BC];
		short data[][] = new short[4][BC];
		short key[][] = new short[4][KC];

		/*
		 * Copy the data and key from their linear buffers.
		 */
		workBuffer = new short[dataBuffer.length];
		astod(dataBuffer, data, 4, BC);
		astod(keyBuffer, key, 4, KC);

		/*
		 * Perform the key scheduling we will use.
		 */
		Rijndael.KeySchedule(key, rk, (short)KC, (short)BC, (short)rounds);
		for(int z = 0; z <= rounds; ++z) {
			Write("rk[" + new Integer(z) + "] = ");
			adtos(workBuffer, rk[z], 4, BC);
			print();
		}

		/*
		 * Perform the step by step encryption, then decryption, via AES.
		 */
		Write("Original:   "); adtos(workBuffer, data, 4, BC); print();
		stepEncrypt(data, rk, (short)BC, (short)rounds);
		Write("Ciphertext: "); adtos(workBuffer, data, 4, BC); print();
		stepDecrypt(data, rk, (short)BC, (short)rounds);
		Write("Plaintext:  "); adtos(workBuffer, data, 4, BC); print();

		reset();
	}


	private void stepEncrypt(short data[][], short rk[][][], short BC, short rounds) {
		/*
		 * Encrypt via the AES standard.
		 */
		AES Rijndael = new AES();
//		short tmp[] = new short[4 * BC];

		/* Start by adding in key 0. */
//		adtos(tmp, data, 4, BC);
//		WriteGUI("Plaintext is:        " + ArraytoHex(tmp) + ".\n");
		Rijndael.KeyAddition(data, rk[0], BC);

		/* Perform (rounds - 1) full rounds. */
		for(int r = 1; r < rounds; ++r) {
			Rijndael.Substitution(data, Boxes.S, BC);
			Rijndael.ShiftRow(data, (short)0, BC);
			Rijndael.MixColumn(data, BC);
			Rijndael.KeyAddition(data, rk[r], BC);
//			adtos(tmp, data, 4, BC);
//			WriteGUI("IV after round " + r + " is: " + ArraytoHex(tmp) + ".\n");
		}

		/* Perform a final round without MixColumn. */
		Rijndael.Substitution(data, Boxes.S, BC);
		Rijndael.ShiftRow(data, (short)0, BC);
		Rijndael.KeyAddition(data, rk[rounds], BC);
//		adtos(tmp, data, 4, BC);
//		WriteGUI("Ciphertext is:       " + ArraytoHex(tmp) + ".\n");

	}


	private void stepDecrypt(short data[][], short rk[][][], short BC, short rounds) {
		/*
		 * Encrypt via the AES standard.
		 */
		AES Rijndael = new AES();
//		short tmp[] = new short[4 * BC];

		/* Perform the inverse of the short round of encryption. */
//		adtos(tmp, data, 4, BC);
//		WriteGUI("Ciphertext is:       " + ArraytoHex(tmp) + ".\n");
		Rijndael.KeyAddition(data, rk[rounds], BC);
		Rijndael.Substitution(data, Boxes.Si, BC);
		Rijndael.ShiftRow(data, (short)1, BC);

		/* Perform (rounds - 1) full rounds. */
		for(int r = rounds - 1; r > 0; --r) {
//			adtos(tmp, data, 4, BC);
//			WriteGUI("IV after round " + (rounds - r) + " is: " + ArraytoHex(tmp) + ".\n");
			Rijndael.KeyAddition(data, rk[r], BC);
			Rijndael.InvMixColumn(data, BC);
			Rijndael.Substitution(data, Boxes.Si, BC);
			Rijndael.ShiftRow(data, (short)1, BC);
		}

		/* End by adding in (cancelling) key 0. */
		Rijndael.KeyAddition(data, rk[0], BC);
//		adtos(tmp, data, 4, BC);
//		WriteGUI("Plaintext is:        " + ArraytoHex(tmp) + ".\n");

	}

}
