/*
 * GUI.java
 * File revision 0
 * GUI for the Crypto system.
 * (c) 2002 Jacob Lundberg, jacob@chaos2.org
 */

/*  Java imports for the applet.  */
import java.awt.*;
import java.applet.*;

/*  The Crypto GUI class.  */
public class GUI extends Applet {

	/*  Discriminators for actions on the second frame.  */
	private final int NOTHING = 0,
			  ENCRYPT = 1,
			  DECRYPT = 2,
			  RESET   = 3;

	private final String EMPTY = new String();

	/*  A button for any need.  */
	private Button btnEncrypt = new Button("Encrypt"),
		       btnDecrypt = new Button("Decrypt"),
		       btnClear   = new Button("Clear"),
		       btnExit    = new Button("Exit"),
		       btnClrHst  = new Button("Clear History");

	/* Switches that control modes of operation. */
	private Checkbox cbxMonteCarlo = new Checkbox(" Monte Carlo Mode", false),
			 cbxUseASCII   = new Checkbox(" Provide ASCII Conversion", false);

	/* Labels for the various text fields. */
	private Label lblASCIItext  = new Label("ASCII Text:", Label.RIGHT),
		      lblPlaintext  = new Label("Plain Text:", Label.RIGHT),
		      lblKeyvalue   = new Label("Key value:", Label.RIGHT),
		      lblCiphertext = new Label("Cipher Text:", Label.RIGHT),
		      lblOperations = new Label("Operations:", Label.RIGHT),
		      lblTitle      = new Label("AES Crypto Applet", Label.CENTER),
		      lblEmpty      = new Label(EMPTY);

	/* Labelled Textfields (input and output). */
	private TextField txtASCIItext  = new TextField(32),
			  txtPlaintext  = new TextField(96),
			  txtKeyvalue   = new TextField(96),
			  txtCiphertext = new TextField(96);

	/* The operations log text area. */
	private TextArea txaOperations = new TextArea(25, 100);

	/* Keep track of what we're doing so button multiclicks don't screw us up. */
	private int whatActive;

	/*  Where the working crypto system will reside.  */
	private Crypto theCrypto;


	/*  It's amasing how often we need to _force_ the awt to rethink its display.  */
	private void reDraw() {
		doLayout();
		validate();
		repaint();
	}


	/* Write a message to the information text box. */
	public void WriteLog(String message) {
		if(whatActive != RESET)
			txaOperations.setText(txaOperations.getText() + message);
	}


	public void init() {
		/*
		 * Prepare the applet for operation.
		 */

		whatActive = NOTHING;
		resize(900, 650);
		reDraw();

		/*
		 * The root panel, which provides the top level layout.
		 */
		Panel rootPanel = new Panel();
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		rootPanel.setLayout(gbl);

		lblTitle.setFont(new Font("Times New Roman", Font.BOLD, 18));
		rootPanel.add(lblTitle);
		gbl.setConstraints(lblTitle, gbc);

		/*
		 * The action menu panel.
		 */
		Panel mainMenu = new Panel();
		mainMenu.setBackground(Color.pink);
		mainMenu.setLayout(new GridLayout(1, 4, 2, 2));
		mainMenu.add(btnClear);
		mainMenu.add(btnEncrypt);
		mainMenu.add(btnDecrypt);
		mainMenu.add(btnExit);
		rootPanel.add(mainMenu);
		gbl.setConstraints(mainMenu, gbc);

		/*
		 * The display and I/O panel.
		 */
		Panel activePanel = new Panel();
		activePanel = new Panel();
		activePanel.setBackground(Color.cyan);
		GridBagLayout gblActive = new GridBagLayout();
		activePanel.setLayout(gblActive);
		GridBagConstraints gbcWest = (GridBagConstraints)gbc.clone();
		gbcWest.anchor = GridBagConstraints.WEST;
		GridBagConstraints gbcEast = (GridBagConstraints)gbc.clone();
		gbcEast.anchor = GridBagConstraints.EAST;

		activePanel.add(cbxUseASCII);
		gblActive.setConstraints(cbxUseASCII, gbcWest);
		activePanel.add(cbxMonteCarlo);
		gblActive.setConstraints(cbxMonteCarlo, gbcWest);

		gbcEast = (GridBagConstraints)gbcEast.clone();
		gbcEast.gridwidth = 1;

		activePanel.add(lblASCIItext);
		gblActive.setConstraints(lblASCIItext, gbcEast);
		activePanel.add(txtASCIItext);
		gblActive.setConstraints(txtASCIItext, gbcWest);

		activePanel.add(lblPlaintext);
		gblActive.setConstraints(lblPlaintext, gbcEast);
		activePanel.add(txtPlaintext);
		gblActive.setConstraints(txtPlaintext, gbcWest);

		activePanel.add(lblKeyvalue);
		gblActive.setConstraints(lblKeyvalue, gbcEast);
		activePanel.add(txtKeyvalue);
		gblActive.setConstraints(txtKeyvalue, gbcWest);

		activePanel.add(lblCiphertext);
		gblActive.setConstraints(lblCiphertext, gbcEast);
		activePanel.add(txtCiphertext);
		gblActive.setConstraints(txtCiphertext, gbcWest);

		activePanel.add(lblEmpty);
		gblActive.setConstraints(lblEmpty, gbc);

		activePanel.add(lblOperations);
		gblActive.setConstraints(lblOperations, gbcEast);
		activePanel.add(btnClrHst);
		gblActive.setConstraints(btnClrHst, gbcWest);
		activePanel.add(txaOperations);
		gblActive.setConstraints(txaOperations, gbc);

		rootPanel.add(activePanel);
		gbl.setConstraints(activePanel, gbc);

		theCrypto = new Crypto();
		theCrypto.gui = this;
		theCrypto.reset();

		add(rootPanel);
		reDraw();

	}


	public boolean action(Event a, Object o) {
		/*
		 * Listen for events.
		 *
		 * Technically we should split this up and
		 * register it as action listeners for each button.
		 */

		if(a.target == btnEncrypt) {
			/* Attempt to encrypt the plaintext. */
			if(whatActive != NOTHING)
				return(true);
			whatActive = ENCRYPT;

			if(cbxUseASCII.getState())
				txtPlaintext.setText(theCrypto.ASCIItoHex(txtASCIItext.getText()));
			String text = txtPlaintext.getText();
			String key  = txtKeyvalue.getText();

			int iter = 1;
			if(cbxMonteCarlo.getState())
				iter = 10000;

			text = theCrypto.encrypt(text, key, iter);

			txtCiphertext.setText(text);

			whatActive = NOTHING;
			return(true);

		} else if(a.target == btnDecrypt) {
			/* Attempt to decrypt the ciphertext. */
			if(whatActive != NOTHING)
				return(true);
			whatActive = DECRYPT;

			String text = txtCiphertext.getText();
			String key  = txtKeyvalue.getText();

			int iter = 1;
			if(cbxMonteCarlo.getState())
				iter = 10000;

			text = theCrypto.decrypt(text, key, iter);

			txtPlaintext.setText(text);

			if(cbxUseASCII.getState())
				txtASCIItext.setText(theCrypto.HextoASCII(txtPlaintext.getText()));

			whatActive = NOTHING;
			return(true);

		} else if(a.target == btnClear) {
			/* Clear all inputs and outputs. */
			if(whatActive != NOTHING)
				return(true);
			whatActive = RESET;

			cbxUseASCII.setState(false);
			cbxMonteCarlo.setState(false);

			txtASCIItext.setText(EMPTY);
			txtPlaintext.setText(EMPTY);
			txtKeyvalue.setText(EMPTY);
			txtCiphertext.setText(EMPTY);

			txaOperations.setText(EMPTY);

			theCrypto.reset();

			reDraw();
			whatActive = NOTHING;
			return(true);


		} else if(a.target == btnClrHst) {
			/* Clear the Operations History. */
			if(whatActive != NOTHING)
				return(true);
			whatActive = RESET;

			txaOperations.setText(EMPTY);

			reDraw();
			whatActive = NOTHING;
			return(true);

		} else if(a.target == btnExit) {
			/* Blow it all away... */
			removeAll();
			return(true);

		} else {
			/* We didn't handle the event (uh oh). */
			return(false);

		}

	}

}
